@@ -15,7 +15,10 @@ function setup_openstack_clients {
         packages+=(python-zunclient)
     fi
     if [[ $SCENARIO == ironic ]]; then
-        packages+=(python-ironicclient)
+        packages+=(python-ironicclient python-ironic-inspector-client)
+    fi
+    if [[ $SCENARIO == magnum ]]; then
+        packages+=(python-designateclient python-magnumclient python-octaviaclient)
     fi
     if [[ $SCENARIO == masakari ]]; then
         packages+=(python-masakariclient)
@@ -23,6 +26,9 @@ function setup_openstack_clients {
     if [[ $SCENARIO == scenario_nfv ]]; then
         packages+=(python-tackerclient python-barbicanclient python-mistralclient)
     fi
+    if [[ $SCENARIO == ovn ]]; then
+        packages+=(python-octaviaclient)
+    fi
     if [[ "debian" == $BASE_DISTRO ]]; then
         sudo apt -y install python3-venv
     fi
@@ -54,11 +60,14 @@ function prepare_images {
     fi
 
     if [[ $SCENARIO == "scenario_nfv" ]]; then
-        GATE_IMAGES+=",^tacker,^mistral,^redis,^barbican"
+        GATE_IMAGES+=",^aodh,^tacker,^mistral,^redis,^barbican"
     fi
     if [[ $SCENARIO == "ironic" ]]; then
         GATE_IMAGES+=",^dnsmasq,^ironic,^iscsid"
     fi
+    if [[ $SCENARIO == "magnum" ]]; then
+        GATE_IMAGES+=",^designate,^magnum,^octavia"
+    fi
     if [[ $SCENARIO == "masakari" ]]; then
         GATE_IMAGES+=",^masakari"
     fi
@@ -68,7 +77,7 @@ function prepare_images {
     fi
 
     if [[ $SCENARIO == "ovn" ]]; then
-        GATE_IMAGES+=",^ovn"
+        GATE_IMAGES+=",^octavia,^ovn"
     fi
 
     if [[ $SCENARIO == "mariadb" ]]; then
